# -*- coding: utf-8 -*-

"""
--------------------------------------------
project: zibuyu_wechat
author: 子不语
date: 2024/11/15
contact: 【公众号】思维兵工厂
description: 数据类型
--------------------------------------------
"""

from typing import Literal, List
from dataclasses import dataclass, field


@dataclass
class Reply:
    """自动回复的内容"""

    reply_type: Literal['text', 'img', 'voice', 'video', 'news'] = None
    content: str = None  # 文本类型的内容

    news_info: str = None  # 图文消息的信息
    title: str = None  # 图文消息的标题
    digest: str = None  # 摘要
    author: str = None  # 作者
    show_cover: bool = False  # 是否显示封面
    cover_url: str = None  # 封面图片的URL
    content_url: str = None  # 正文的URL
    source_url: str = None  # 原文的URL，若置空则无查看原文入口


@dataclass
class Keyword:
    """自动回复的关键字"""

    keyword_type: Literal['text', 'img', 'voice', 'video', 'news'] = None
    match_mode: Literal['contain', 'equal'] = None
    content: str = None  # 文本类型的内容


@dataclass
class KeywordInfo:
    """关键字回复数据类"""

    rule_name: str = ''  # 规则名称
    create_time: int = None  # 规则创建时间
    reply_mode: Literal['reply_all', 'random_one'] = None  # 回复模式，reply_all代表全部回复，random_one代表随机回复其中一条
    keyword_list: list[Keyword] = None  # 触发关键词列表
    reply_list: list[Reply] = None  # 回复内容列表


@dataclass
class WechatOfficialKeyword:
    """公众号自动回复数据类"""

    # 关注后自动回复是否开启
    is_add_friend_reply: bool = False

    # 消息自动回复是否开启
    is_auto_reply: bool = False

    # 自动回复相关
    add_friend_reply_type: Literal['text', 'img', 'voice', 'video', 'news'] = None

    # 关注后自动回复的信息：对于文本类型，content是文本内容，对于图文、图片、语音、视频类型，content是mediaID
    add_friend_reply_content: str = None

    # 自动回复的类型。关注后自动回复和消息自动回复的类型仅支持文本（text）、图片（img）、语音（voice）、视频（video），关键词自动回复则多了图文消息（news）
    add_friend_reply_media_id: str = None

    # 自动回复列表
    keyword_reply_info_list: list[KeywordInfo] = None


@dataclass
class QuotaResult:
    """公众号接口调用额度"""
    daily_limit: int  # 当天该账号可调用该接口的次数
    used: int  # 当天已经调用的次数
    remain: int  # 当天剩余调用次数


@dataclass
class NewsItem:
    """公众号素材：图文消息中的item"""
    title: str
    thumb_media_id: str
    show_cover_pic: Literal[0, 1]
    author: str
    digest: str
    content: str
    url: str
    content_source_url: str


@dataclass
class SourceNewsItem:
    """公众号素材单项：图文消息"""
    media_id: str
    update_time: str
    item_list: List[NewsItem]


@dataclass
class CommonItem:
    """公众号素材单项：图片、语音、视频"""
    media_id: str
    name: str = ''
    update_time: str = ''
    url: str = ''
    tags: str = ''


@dataclass
class SourceNewsItemList:
    """公众号素材：图文消息"""
    total_count: int
    item_count: int
    item_list: List[SourceNewsItem]


@dataclass
class SourceCommonItemList:
    """公众号素材：图片、语音、视频"""

    type: Literal['image', 'voice', 'video']
    total_count: int
    item_count: int
    item_list: List[CommonItem]


@dataclass
class SourceCount:
    """公众号素材总数"""

    voice_count: int
    video_count: int
    image_count: int
    news_count: int


@dataclass
class Draft:
    """微信公众号草稿"""

    title: str  # 草稿标题，必填
    content: str  # 草稿内容，必填
    thumb_media_id: str  # 封面图片素材id（必须是永久MediaID），必填

    author: str = ''  # 作者，非必填
    digest: str = ''  # 图文摘要；仅单图文消息才有摘要，多图文此处为空。如果本字段为没有填写，则默认抓取正文前54个字。

    content_source_url: str = ''  # 图文消息的原文地址，即点击“阅读原文”后的URL
    need_open_comment: Literal[0, 1] = 0  # 是否打开评论；0不打开(默认)，1打开
    only_fans_can_comment: Literal[0, 1] = 0  # 是否粉丝才可评论；0所有人可评论(默认)，1粉丝才可评论

    pic_crop_235_1: str = ''  # 草稿封面裁剪，非必填
    pic_crop_1_1: str = ''  # 草稿封面裁剪，非必填

    # 获取草稿时才有的字段
    url: str = ''  # 草稿的临时链接
    media_id: str = ''  # 草稿的media_id
    update_time: int = -1  # 草稿的更新时间

    # 发表成为正式文章后才有的字段
    is_deleted: bool = False  # 是否已删除
    show_cover_pic: Literal[0, 1] = 0  # 是否显示封面，0为false，即不显示，1为true，即显示(默认)
    thumb_url: str = ''  # 首图地址

    def to_dict(self) -> dict:
        return {
            'title': self.title,
            'content': self.content,
            'thumb_media_id': self.thumb_media_id,
            'author': self.author,
            'digest': self.digest,
            'content_source_url': self.content_source_url,
            'need_open_comment': self.need_open_comment,
            'only_fans_can_comment': self.only_fans_can_comment,
            'pic_crop_235_1': self.pic_crop_235_1,
            'pic_crop_1_1': self.pic_crop_1_1
        }


@dataclass
class DraftList:
    total_count: int
    item_count: int
    draft_list: List[Draft]


@dataclass
class ArticleDetailItem:
    """文章详情条目"""
    idx: int = ''  # 文章索引
    article_url: str = ''  # 文章URL


@dataclass
class ArticleDetail:
    """文章详情"""
    item_list: List[ArticleDetailItem]  # 文章详情列表，默认为None
    count: int = 0  # 文章数量，默认为0


@dataclass
class ArticlePublishStatus:
    """文章发布状态"""
    publish_id: str  # 发布任务id
    publish_status: int  # 发布状态，0:成功, 1:发布中, 2:原创失败, 3:常规失败, 4:平台审核不通过, 5:成功后用户删除所有文章, 6:成功后系统封禁所有文章

    article_detail: ArticleDetail  # 文章详情，默认为None

    article_id: str = ''  # 当发布状态为0时（即成功）时，返回图文的article_id，否则为None
    fail_idx: List[int] = field(default_factory=list)  # 失败的文章索引列表，默认为空列表
